"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthCallQueryRequest = void 0;
const BinaryWriter_1 = require("./BinaryWriter");
const request_1 = require("./request");
const utils_1 = require("./utils");
class EthCallQueryRequest {
    constructor(blockTag, callData) {
        this.callData = callData;
        if (typeof blockTag === "number") {
            this.blockTag = `0x${blockTag.toString(16)}`;
        }
        else if ((0, utils_1.isValidHexString)(blockTag)) {
            if (!blockTag.startsWith("0x")) {
                blockTag = `0x${blockTag}`;
            }
            this.blockTag = blockTag;
        }
        else {
            throw new Error(`Invalid block tag: ${blockTag}`);
        }
    }
    type() {
        return request_1.ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint32(this.blockTag.length)
            .writeUint8Array(Buffer.from(this.blockTag))
            .writeUint8(this.callData.length);
        this.callData.forEach(({ to, data }) => {
            const dataArray = (0, utils_1.hexToUint8Array)(data);
            writer
                .writeUint8Array((0, utils_1.hexToUint8Array)(to))
                .writeUint32(dataArray.length)
                .writeUint8Array(dataArray);
        });
        return writer.data();
    }
}
exports.EthCallQueryRequest = EthCallQueryRequest;
