"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthCallByTimestampQueryRequest = void 0;
const BinaryWriter_1 = require("./BinaryWriter");
const request_1 = require("./request");
const utils_1 = require("./utils");
class EthCallByTimestampQueryRequest {
    constructor(targetTimestamp, targetBlockHint, followingBlockHint, callData) {
        this.callData = callData;
        this.targetTimestamp = targetTimestamp;
        this.targetBlockHint = parseBlockHint(targetBlockHint);
        this.followingBlockHint = parseBlockHint(followingBlockHint);
    }
    type() {
        return request_1.ChainQueryType.EthCallByTimeStamp;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint64(this.targetTimestamp)
            .writeUint32(this.targetBlockHint.length)
            .writeUint8Array(Buffer.from(this.targetBlockHint))
            .writeUint32(this.followingBlockHint.length)
            .writeUint8Array(Buffer.from(this.followingBlockHint))
            .writeUint8(this.callData.length);
        this.callData.forEach(({ to, data }) => {
            const dataArray = (0, utils_1.hexToUint8Array)(data);
            writer
                .writeUint8Array((0, utils_1.hexToUint8Array)(to))
                .writeUint32(dataArray.length)
                .writeUint8Array(dataArray);
        });
        return writer.data();
    }
}
exports.EthCallByTimestampQueryRequest = EthCallByTimestampQueryRequest;
function parseBlockHint(blockHint) {
    // Block hints are not required.
    if (blockHint != "") {
        if (typeof blockHint === "number") {
            blockHint = `0x${blockHint.toString(16)}`;
        }
        else if ((0, utils_1.isValidHexString)(blockHint)) {
            if (!blockHint.startsWith("0x")) {
                blockHint = `0x${blockHint}`;
            }
            blockHint = blockHint;
        }
        else {
            throw new Error(`Invalid block tag: ${blockHint}`);
        }
    }
    return blockHint;
}
