"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChainQueryType = exports.PerChainQueryRequest = exports.QueryRequest = exports.getPrefix = exports.DEVNET_QUERY_REQUEST_PREFIX = exports.TESTNET_QUERY_REQUEST_PREFIX = exports.MAINNET_QUERY_REQUEST_PREFIX = void 0;
const BinaryWriter_1 = require("./BinaryWriter");
const web3_1 = require("web3");
const utils_1 = require("./utils");
exports.MAINNET_QUERY_REQUEST_PREFIX = "mainnet_query_request_000000000000|";
exports.TESTNET_QUERY_REQUEST_PREFIX = "testnet_query_request_000000000000|";
exports.DEVNET_QUERY_REQUEST_PREFIX = "devnet_query_request_0000000000000|";
function getPrefix(network) {
    return network === "MAINNET"
        ? exports.MAINNET_QUERY_REQUEST_PREFIX
        : network === "TESTNET"
            ? exports.TESTNET_QUERY_REQUEST_PREFIX
            : exports.DEVNET_QUERY_REQUEST_PREFIX;
}
exports.getPrefix = getPrefix;
class QueryRequest {
    constructor(nonce, requests = [], version = 1) {
        this.nonce = nonce;
        this.requests = requests;
        this.version = version;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint8(this.version)
            .writeUint32(this.nonce)
            .writeUint8(this.requests.length);
        this.requests.forEach((request) => writer.writeUint8Array(request.serialize()));
        return writer.data();
    }
    static digest(network, bytes) {
        const prefix = getPrefix(network);
        const data = Buffer.concat([Buffer.from(prefix), bytes]);
        return (0, utils_1.hexToUint8Array)(web3_1.utils.keccak256(data).slice(2));
    }
}
exports.QueryRequest = QueryRequest;
class PerChainQueryRequest {
    constructor(chainId, query) {
        this.chainId = chainId;
        this.query = query;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint16(this.chainId)
            .writeUint8(this.query.type());
        const queryData = this.query.serialize();
        return writer
            .writeUint32(queryData.length)
            .writeUint8Array(queryData)
            .data();
    }
}
exports.PerChainQueryRequest = PerChainQueryRequest;
var ChainQueryType;
(function (ChainQueryType) {
    ChainQueryType[ChainQueryType["EthCall"] = 1] = "EthCall";
    ChainQueryType[ChainQueryType["EthCallByTimeStamp"] = 2] = "EthCallByTimeStamp";
})(ChainQueryType || (exports.ChainQueryType = ChainQueryType = {}));
