import { BinaryWriter } from "./BinaryWriter";
import { ChainQueryType } from "./request";
import { hexToUint8Array, isValidHexString } from "./utils";
export class EthCallQueryRequest {
    constructor(blockTag, callData) {
        this.callData = callData;
        if (typeof blockTag === "number") {
            this.blockTag = `0x${blockTag.toString(16)}`;
        }
        else if (isValidHexString(blockTag)) {
            if (!blockTag.startsWith("0x")) {
                blockTag = `0x${blockTag}`;
            }
            this.blockTag = blockTag;
        }
        else {
            throw new Error(`Invalid block tag: ${blockTag}`);
        }
    }
    type() {
        return ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter()
            .writeUint32(this.blockTag.length)
            .writeUint8Array(Buffer.from(this.blockTag))
            .writeUint8(this.callData.length);
        this.callData.forEach(({ to, data }) => {
            const dataArray = hexToUint8Array(data);
            writer
                .writeUint8Array(hexToUint8Array(to))
                .writeUint32(dataArray.length)
                .writeUint8Array(dataArray);
        });
        return writer.data();
    }
}
