import * as elliptic from "elliptic";
export function isValidHexString(s) {
    return /^(0x)?[0-9a-fA-F]+$/.test(s);
}
export function hexToUint8Array(s) {
    if (!isValidHexString(s)) {
        throw new Error(`${s} is not hex`);
    }
    if (s.startsWith("0x")) {
        s = s.slice(2);
    }
    s.padStart(s.length + (s.length % 2), "0");
    return new Uint8Array(Buffer.from(s, "hex"));
}
/**
 * @param key Private key used to sign `data`
 * @param data Data for signing
 * @returns ECDSA secp256k1 signature
 */
export function sign(key, data) {
    var _a;
    const ec = new elliptic.ec("secp256k1");
    const keyPair = ec.keyFromPrivate(key);
    const signature = keyPair.sign(data, { canonical: true });
    const packed = signature.r.toString("hex").padStart(64, "0") +
        signature.s.toString("hex").padStart(64, "0") +
        Buffer.from([(_a = signature.recoveryParam) !== null && _a !== void 0 ? _a : 0]).toString("hex");
    return packed;
}
